#include <Demos/demos.h>
#include <Demos/Physics/Api/MyTest/MyTestDemo.h>

MyTestDemo::MyTestDemo(hkDemoEnvironment* env)
:	hkDefaultPhysicsDemo(env)
{
	// configuration de la camra
	{
		hkVector4 from(0.0f, 7.0f, 25.0f);
		hkVector4 to  (0.0f, 3.0f,  0.0f);
		hkVector4 up  (0.0f, 1.0f,  0.0f);
		setupDefaultCameras( env, from, to, up );
	}

	// cration du monde
	{
		hkpWorldCinfo info;
		info.setupSolverInfo(hkpWorldCinfo::SOLVER_TYPE_4ITERS_MEDIUM); 
		info.setBroadPhaseWorldSize( 100.0f );
		// seuil de tolrance pour les collisions, les units tant celle du systme international
		// on obtient donc une tolrance de 4cm
		info.m_collisionTolerance = 0.04f;
		info.m_contactRestingVelocity = 0.2f;
		m_world = new hkpWorld( info );
		m_world->lock();

		setupGraphics();
	}

	{
		hkpAgentRegisterUtil::registerAllAgents(m_world->getCollisionDispatcher());
	}

	// le sol
	{
		// pour dfinir une boite (aligne sur les axes) on donne un vecteur
		// correspondant  une demi-diagonale, donc ici, notre boite
		//  mesurera  100.0f x 2.0f x 100.0f
		hkVector4 demiDiagonaleDuSol( 50.0f, 1.0f, 50.0f );

		// on cr lobjet qui va dfinir notre boite
		hkpConvexShape *solide = new hkpBoxShape( demiDiagonaleDuSol, 0.0f );

		// cet objet contient des informations  propos de notre solide physique
		hkpRigidBodyCinfo ci;

		// on spcifie ici que notre solide physique aura les dimensions
		// de notre boite ci-dessus
		ci.m_shape = solide;

		// notre solide sera fixe dans notre monde
		ci.m_motionType = hkpMotion::MOTION_FIXED;

		// position du centre de notre solide
		ci.m_position = hkVector4( 0.0f, -1.0f, 0.0f );

		// spcifie  lalgo  qui sera utilis pour tester les collisions
		// avec notre solide
		ci.m_qualityType = HK_COLLIDABLE_QUALITY_FIXED;

		// ajoute le solide au monde et libre les objets devenus inutiles
		m_world->addEntity( new hkpRigidBody( ci ) )->removeReference();
		solide->removeReference();
	}

	// la sphere
	{
		// la classe pour les proprits de notre future sphre
		hkpRigidBodyCinfo sphereInfo;

		// attention cest technique, ici on calcule le centre de gravit, ainsi quune
		// matrice qui va indiquer comment va se comporter notre objet lors de rotations
		// sur lui-mme, grosso modo comment se rparti la masse de notre objet
		// autour de son centre de gravit
		// notre sphre pse 100.0f et a pour rayon 1.0f
		hkpMassProperties massProperties;
		hkpInertiaTensorComputer::computeSphereVolumeMassProperties(1.0f, 100.0f, massProperties);

		// remplissage des informations sur notre sphre
		sphereInfo.m_mass = massProperties.m_mass;
		sphereInfo.m_centerOfMass  = massProperties.m_centerOfMass;
		sphereInfo.m_inertiaTensor = massProperties.m_inertiaTensor;
		sphereInfo.m_shape = new hkpSphereShape( 1.0f );
		sphereInfo.m_position = hkVector4( 0.0f, 10.0f, 0.0f );
		sphereInfo.m_motionType = hkpMotion::MOTION_SPHERE_INERTIA;

		// voici un nouvel  algo  de collision, celui-ci permet de ne pas louper
		// de collision quand un des objets peut avoir une vitesse leve
		sphereInfo.m_qualityType = HK_COLLIDABLE_QUALITY_BULLET;

		// cration de notre objet avec toutes ses caractristiques
		hkpRigidBody* sphereRigidBody = new hkpRigidBody( sphereInfo );

		// ajout de notre objet  la scne et libration des ressources
		m_world->addEntity( sphereRigidBody );
		sphereRigidBody->removeReference();
		sphereInfo.m_shape->removeReference();
	}

	// la boite
	{
		// quelque soit lobjet que lon va crer, on rempli encore la classe
		// hkpRigidBodyCinfo
		hkpRigidBodyCinfo boxInfo;

		// on renseigne la masse de notre boite
		boxInfo.m_mass = 10.0f;

		hkVector4 demiDiagonaleDeLaBoite( 2.0f, 2.0f, 2.0f );

		hkpConvexShape *solide = new hkpBoxShape( demiDiagonaleDeLaBoite, 0.0f );

		// de la mme manire que pour la sphre on calcule des proprits
		// avances pour notre boite
		hkpMassProperties massProperties;
		hkpInertiaTensorComputer::computeBoxVolumeMassProperties(demiDiagonaleDeLaBoite, boxInfo.m_mass, massProperties);

		boxInfo.m_mass = massProperties.m_mass;
		boxInfo.m_centerOfMass = massProperties.m_centerOfMass;
		boxInfo.m_inertiaTensor = massProperties.m_inertiaTensor;
		boxInfo.m_shape = solide;

		// voici un champ quon a pas encore vu, celui-ci (compris entre 0.0 et 1.0)
		// permet de calculer la quantit dnergie qui sera conserve
		// lors de la collision entre notre boite et un autre objet de la scne
		boxInfo.m_restitution = 1.0f;

		boxInfo.m_motionType = hkpMotion::MOTION_BOX_INERTIA;

		boxInfo.m_position = hkVector4( 0.1f, 15.0f, 0.0f );
		hkpRigidBody* boxRigidBody = new hkpRigidBody( boxInfo );
		m_world->addEntity( boxRigidBody );
		boxRigidBody->removeReference();
		boxInfo.m_shape->removeReference();
	}

	m_world->unlock();
}

#if defined(HK_COMPILER_MWERKS)
#	pragma force_active on
#	pragma fullpath_file on
#endif

static const char helpString[] = "une simple application de test.";

HK_DECLARE_DEMO(MyTestDemo, HK_DEMO_TYPE_PRIME, "notre demo", helpString);
